# COMPARE NEYMAN TYPE A MODEL (1) AND NEYMAN TYPE A MODEL (2) FOR LEADBEATER'S POSSUM 
# COUNTS

options(contrasts=c("contr.treatment","contr.poly"),object.size=1E8)


# READ IN DATA. SET UP COVARIATES i.e. TRANSFORMATION, MEAN CORRECTION, 
# FACTORISATION, ETC.
counts <- read.table("data.txt",sep="\t",header=T,row.names=NULL,col.names=c("no.lb","stags","slope","baa","shrubs","bark"))
attach(counts)

lstags <- log(stags+1) 


# SET UP THE RESPONSE AND DESIGN MATRICES FROM AN INPUT FORMULA
"setup"<-
function(formula)
{
	call <- match.call()
        m <- match.call(expand = F)
        m[[1]] <- as.name("model.frame")
        m <- eval(m, sys.parent())
        Terms <- attr(m, "terms")
        y <- as.matrix(model.extract(m, response))
        x <- model.matrix(Terms, m)
	avg.count <- mean(y)
        var.count <- var(y)
        index <- var.count/avg.count
	if(index <= 1) {
		print("Unable to fit the Neyman Distribution")
		stop
	}
	list(y=y,x=x)
}


# CALCULATE INITIAL ESTIMATES FOR THE TWO PARAMETERS, BETA AND ALPHA
"slr" <- 
function(y,x,z)
{
	print(ncol(x))
	if (ncol(x) > 1) newx <- as.matrix(x[,-1]) # remove column of ones
	else newx <- x
	print(ncol(z))
	if (ncol(z) > 1) newz <- as.matrix(z[,-1]) # remove column of ones
	else newz <- z
	lr <- glm(y~newx+newz,family=poisson,link=log)
	lr$coefficients
}


# CALCULATE THE RATIO OF THE (r+1)th to the rth UNCORRECTED 
# MOMENT OF A POISSON DISTRIBUTION WITH MEAN mu. 
"mubit" <- 
function(mu,r)
{
# calculation of the (r+1)th uncorrected moment (=mu1)
	delta1 <- 1 #initialisation of maximum increment
        len <- length(mu)
        del1 <- rep(0,len) #initialisation of increment
        jj <- 1 #sum counter
        mu1 <- rep(0,len) #initialisation of mu1
        while (delta1 > 0.000001 | jj <= 10) {
                del1 <- exp(-mu)*(mu^jj)/gamma(jj+1)*(jj^(r+1))
                mu1 <- mu1 + del1
                delta1 <- max(abs(del1))
                jj <- jj + 1
        }

        # calculation of the rth uncorrected moment (=mu0)
        delta0 <- 1 #initialisation of maximum increment
        del0 <- 0 #initialisation of increment
        kk <- 1 #sum counter
        mu0 <- rep(0,len) #initialisation of mu1
        for (dd in c(1:len)) {
                if (r[dd]==0) mu0[dd] <- 1
                else {
			while (delta0 > 0.000001 | kk <= 10) {
                                del0 <- exp(-mu[dd])*(mu[dd]^kk)/gamma(kk+1)*(kk^r[dd])
                                mu0[dd] <- mu0[dd] + del0
                                delta0 <- abs(del0)
                                kk <- kk + 1
			}
                }
                del0 <- 0
                delta0 <- 1
                kk <- 1
        }
	res <- mu1/mu0
        res
}


# CALCULATE THE LOG-LIKELIHOOD FOR THE NEYMAN TYPE A MODEL (1)
"loglik.null" <-
function(lambda,phi,y)
{
	n <- length(y)
	sum <- 0
	innersum <- rep(0,n)
	for (i in c(1:n)) {
		j <- 0
		delta <- 1
		if (y[i]==0) innersum[i] <- exp(lambda[i]*exp(-phi[i]))
		else while (delta > 0.0000001 | j <= 10) {
			j <- j + 1
			delta <- (lambda[i] * exp(-phi[i]))^j * (j^y[i]) / gamma(j+1)
			innersum[i] <- innersum[i] + delta
		}
		sum <- sum - lambda[i] + y[i] * log(phi[i]) - log(gamma(y[i] + 1)) + 
			log(innersum[i])
	}
	sum
}

# CALCULATE THE LOG-LIKELIHOOD FOR THE NEYMAN TYPE A MODEL (2)
"loglik.alt" <-
function(psi,phi,y)
{
        n <- length(y)
        sum <- 0
        innersum <- rep(0,n)
        for (i in c(1:n)) {
                j <- 0
                delta <- 1
                if (y[i]==0) innersum[i] <- exp(psi[i])
                else while (delta > 0.0000001 | j <= 10) {
                        j <- j + 1
                        delta <- (psi[i])^j * (j^y[i]) / gamma(j+1)
                        innersum[i] <- innersum[i] + delta
                }
                sum <- sum - psi[i] * exp(phi[i]) + y[i] * log(phi[i]) -
                        log(gamma(y[i] + 1)) + log(innersum[i])
        }
        sum
}

# CALCULATE THE EXPECTED VALUE UNDER NULL HYPOTHESIS OF NULL LOG-LIKELIHOOD
# MINUS ALTERNATIVE LOG-LIKELIHOOD
"exp.fg" <-
function(lamN,phiN,psiA,phiA,n)
{
	sum <- 0
	sum2 <- 0
	for (i in c(1:n)) {
		newsum <- 0
     		y <- 0
                outer <- 1
                while (abs(outer) > 0.00000001 | y <= 10) {
                        innersum <- 0
                        innersum2 <- 0
                        isum <- 0
                        if (y==0) {
                                isum <- exp(lamN[i]*exp(-phiN[i]))
                                fden <- exp(-lamN[i]) * isum
                                fll <- -lamN[i] + log(isum)
                                gll <- -psiA[i] * exp(phiA[i]) + psiA[i]
                                outer <- fden * (fll - gll)
                                sum <- sum + outer
				newsum <- newsum + outer
                        }
                        else {
                                j <- 0
                                delta <- 1
                                while (delta > 0.00000001 | j <= 10) {
                                        j <- j + 1
                                        delta <- (lamN[i] * exp(-phiN[i]))^j * (j^y)/gamma(j+1)
                                        innersum <- innersum + delta
					delta2 <- (psiA[i])^j * (j^y)/gamma(j+1)
                                        innersum2 <- innersum2 + delta2
                                }
                                fden <- exp(-lamN[i]) * phiN[i]^y * innersum /gamma(y+1)
                                fll <- -lamN[i] + y * log(phiN[i]) - log(gamma(y+1)) +
                                        log(innersum)
				gll <- -psiA[i] * exp(phiA[i]) +
                                        y * log(phiA[i]) - log(gamma(y+1)) + log(innersum2)
                                outer <- fden * (fll - gll)
                                sum <- sum + outer
                                newsum <- newsum + outer
                        }
                        y <- y + 1
                }
		sum2 <- sum2 + newsum^2
        }
        list(t1=sum,t2=sum2)
}


# CALCULATE THE EXPECTED VALUE UNDER NULL HYPOTHESIS OF THE SQUARE OF (NULL LOG-LIKELIHOOD
# MINUS ALTERNATIVE LOG-LIKELIHOOD)
"exp.fg2" <-
function(lamN,phiN,psiA,phiA,n)
{
        sum <- 0
        for (i in c(1:n)) {
                y <- 0
                outer <- 1
                while (abs(outer) > 0.00000001 | y <= 10) {
                        innersum <- 0
                        innersum2 <- 0
                        isum <- 0
                        if (y==0) {
                                isum <- exp(lamN[i]*exp(-phiN[i]))
                                fden <- exp(-lamN[i]) * isum
                                fll <- -lamN[i] + log(isum)
				gll <- -psiA[i] * exp(phiA[i]) + psiA[i]
                                outer <- fden * (fll - gll)^2
                                sum <- sum + outer
                        }
                        else {
                                j <- 0
                                delta <- 1
                                while (delta > 0.00000001 | j <= 10) {
                                        j <- j + 1
                                        delta <- (lamN[i] * exp(-phiN[i]))^j * (j^y)/gamma(j+1)
                                        innersum <- innersum + delta
					delta2 <- (psiA[i])^j * (j^y)/gamma(j+1)
                                        innersum2 <- innersum2 + delta2

                                }
                                fden <- exp(-lamN[i]) * phiN[i]^y * innersum /gamma(y+1)
                                fll <- -lamN[i] + y * log(phiN[i]) - log(gamma(y+1)) +
                                        log(innersum)
				gll <- -psiA[i] * exp(phiA[i]) +
                                        y * log(phiA[i]) - log(gamma(y+1)) + log(innersum2)
                                outer <- fden * (fll - gll)^2
                                sum <- sum + outer
                        }
                        y <- y + 1
                }
        }
	sum
}


# CALCULATE THE EXPECTED VALUE UNDER NULL HYPOTHESIS OF (NULL LOG-LIKELIHOOD 
# MINUS ALTERNATIVE LOG-LIKELIHOOD) MULTIPLIED BY THE FIRST DERIVATIVE OF THE NULL 
# LOG-LIKELIHOOD
"exp.fgdLf" <-
function(xN,zN,lamN,phiN,psiA,phiA,n,numN)
{
	pmu <- lamN * exp(-phiN)
        sum <- matrix(0,nrow=numN,ncol=1)

        for (i in c(1:n)) {
               	y <- 0
               	outer <- matrix(1,nrow=numN,ncol=1)
               	while (max(abs(outer)) > 0.00000001 | y <= 10) {
                       	innersum <- 0
                       	innersum2 <- 0
                       	if (y==0) {
                               	isum <- exp(pmu[i])
                               	fden <- exp(-lamN[i]) * isum
                               	fll <- -lamN[i] + log(isum)
				gll <- -psiA[i] * exp(phiA[i]) + psiA[i]
				mubit1 <- pmu[i]
				ub <- as.matrix(xN[i,] * (mubit1 - lamN[i]))
				vb <- as.matrix(-zN[i,] * mubit1 * phiN[i]) 
				dfll <- as.matrix(rbind(ub,vb))
                               	outer <- as.matrix(fden * (fll - gll) * dfll)
                               	sum <- sum + outer
                       	}
                       	else {
                               	k <- 0
                               	delta <- 1
                               	while (delta > 0.00000001 | k <= 10) {
                                       	k <- k + 1
                                       	delta <- (lamN[i] * exp(-phiN[i]))^k * (k^y)/gamma(k+1)
                                       	innersum <- innersum + delta
					delta2 <- (psiA[i])^k * (k^y)/gamma(k+1)
                                        innersum2 <- innersum2 + delta2

                               	}
				mubit1 <- mubit(pmu[i],y)
				if (mubit1 > 10.0) mubit1 <- 10
                               	fden <- exp(-lamN[i]) * phiN[i]^y * innersum /gamma(y+1)
                               	fll <- -lamN[i] + y * log(phiN[i]) - log(gamma(y+1)) + log(innersum)
				gll <- -psiA[i] * exp(phiA[i]) +
                                        y * log(phiA[i]) - log(gamma(y+1)) + log(innersum2)
				ub <- as.matrix(xN[i,] * (mubit1 - lamN[i]))
                                vb <- as.matrix(zN[i,] * (y - mubit1 * phiN[i]))
				dfll <- as.matrix(rbind(ub,vb))
                               	outer <- as.matrix(fden * (fll - gll) * dfll)
                               	sum <- sum + outer
                       	}
                       	y <- y + 1
               	}
       	}
       	sum
}


# CALCULATE THE VARIANCE OF THE FIRST DERIVATIVE OF THE NULL LOG-LIKELIHOOD
"var.dLf" <-
function(xN,zN,lamN,phiN,n,numN)
{
        pmu <- lamN * exp(-phiN)
        sum <- matrix(0,nrow=numN,ncol=numN)

        for (i in c(1:n)) {
		# CALCULATE THE FIRST PART OF THE VARIANCE: E(LL*LL)
		y <- 0
        	sum.big <- matrix(0,nrow=numN,ncol=numN)
        	sum.small <- matrix(0,nrow=numN,ncol=1)
                outer.big <- matrix(1,nrow=numN,ncol=numN)
                while (max(abs(outer.big)) > 0.0000001 | y <= 10) {
                	innersum <- 0
                        if (y==0) {
                                isum <- exp(lamN[i]*exp(-phiN[i]))
                                fden <- exp(-lamN[i]) * isum
				mubit1 <- pmu[i]
                                ub <- as.matrix(xN[i,] * (mubit1 - lamN[i]))
                                vb <- as.matrix(-zN[i,] * mubit1 * phiN[i])
                                dfll <- as.matrix(rbind(ub,vb))
                                outer.big <- fden * dfll %*% t(dfll)
                                sum.big <- sum.big + outer.big
                        }
                        else {
                                j <- 0
                                delta <- 1
                                while (delta > 0.00000001 | j <= 10) {
                                	j <- j + 1
                                        delta <- (lamN[i] * exp(-phiN[i]))^j * (j^y)/gamma(j+1)
                                        innersum <- innersum + delta
                                }
				mubit1 <- mubit(pmu[i],y)
				if (mubit1 > 10) mubit1 <- 10
                                fden <- exp(-lamN[i]) * phiN[i]^y * innersum /gamma(y+1)
                                ub <- as.matrix(xN[i,] * (mubit1 - lamN[i]))
                                vb <- as.matrix(zN[i,] * (y - mubit1 * phiN[i]))
                                dfll <- as.matrix(rbind(ub,vb))
                                outer.big <- fden * dfll %*% t(dfll)
                                sum.big <- sum.big + outer.big
                        }
                        y <- y + 1
                }
		# CALCULATE THE SECOND PART OF THE VARIANCE: E(LL)*E(LL)
		y <- 0
                outer.small <- matrix(1,nrow=numN,ncol=1)
                while (max(abs(outer.small)) > 0.0000001 | y <= 10) {
                        innersum <- 0
                        if (y==0) {
                                isum <- exp(lamN[i]*exp(-phiN[i]))
                                fden <- exp(-lamN[i]) * isum
                                mubit1 <- pmu[i]
                                ub <- as.matrix(xN[i,] * (mubit1 - lamN[i]))
                                vb <- as.matrix(-zN[i,] * mubit1 * phiN[i])
                                dfll <- as.matrix(rbind(ub,vb))
                                outer.small <- fden * dfll
                                sum.small <- sum.small + outer.small
                        }
                        else {
                                j <- 0
                                delta <- 1
                                while (delta > 0.00000001 | j <= 10) {
                                        j <- j + 1
                                        delta <- (lamN[i] * exp(-phiN[i]))^j * (j^y)/gamma(j+1)
                                        innersum <- innersum + delta
                                }
                                mubit1 <- mubit(pmu[i],y)
				if (mubit1 > 10.0) mubit1 <- 10
                                fden <- exp(-lamN[i]) * phiN[i]^y * innersum /gamma(y+1)
                                ub <- as.matrix(xN[i,] * (mubit1 - lamN[i]))
                                vb <- as.matrix(zN[i,] * (y - mubit1 * phiN[i]))
                                dfll <- as.matrix(rbind(ub,vb))
                                outer.small <- fden * dfll 
                                sum.small <- sum.small + outer.small
                        }
                        y <- y + 1
                }
		sum <- sum + sum.big - sum.small %*% t(sum.small)
        }
        sum
}


# CALCULATE ESTIMATES OF BETA AND ALPHA, AND HENCE LAMBDA AND PHI, USING THE
# NEWTON-RAPHSON ALGORITHM - NEYMAN TYPE A MODEL (1)
"NRalg" <-
function(y,x,z,b0,a0)
{
	p <- ncol(x)
	q <- ncol(z)
	phi <- as.vector(exp(z %*% a0))
	phi[phi > 10] <- 10
	phi[phi < 0.001] <- 0.001
	lambda <- as.vector(exp(x %*% b0))
	lambda[lambda < 0.001] <- 0.001
	lambda[lambda > 10] <- 10
	b1 <- matrix(0,nrow=p,ncol=1)
	a1 <- matrix(0,nrow=q,ncol=1)
	tol <- 0.0000001
	maxiter.inner <- 1
	maxiter.outer <- 50
	n <- length(y)
	iter.outer <- 0
	dela <- 1
	delb <- 1
        # CALCULATE POISMU, MUBITS1 AND MUBITS2
        poismu <- lambda * exp(-phi)  #gives an nx1 vector
        mubits1 <- mubit(poismu,y)
        mubits2 <- mubit(poismu,(y+1))

	while (delb > tol & dela > tol & iter.outer < maxiter.outer) {
		iterb <- 0
		itera <- 0
		iter.outer <- iter.outer + 1
		dela <- 1
        	delb <- 1

        	# ITERATION LOOP FOR ALPHA
        	while (dela > tol & itera < maxiter.inner) {
                	itera <- itera + 1
                	print(paste("Starting alpha iteration",iter.outer,"with coefficients"))
                	print(a0)
                	sumpart <- matrix(0,nrow=p,ncol=p)
                	ee.alpha <- matrix(0,nrow=p,ncol=1)

                	scal1 <- phi * (phi * mubits1 * (mubits2 - mubits1) - mubits1)
                	sumpart <- t(z) %*% diag(scal1) %*% z
                	scal2 <- y - mubits1 * phi
                	ee.alpha <- t(z) %*% scal2

                	ee.alpha <- ee.alpha/n
                	fish.alpha <- -sumpart/n
                	dela2 <- solve(fish.alpha) %*% ee.alpha
                	dela <- max(abs(as.vector(dela2)))
                	a1 <- a0 + dela2
                	a0 <- a1
        	}
        	phi <- as.vector(exp(z %*% a1))
		phi[phi > 10] <- 10
		phi[phi < 0.001] <- 0.001
        	poismu <- lambda * exp(-phi)  #gives an nx1 vector
        	mubits1 <- mubit(poismu,y)
        	mubits2 <- mubit(poismu,(y+1))

        	# ITERATION LOOP FOR BETA
        	while (delb > tol & iterb < maxiter.inner) {
                	iterb <- iterb + 1
                	print(paste("Starting beta iteration",iter.outer,"with coefficients"))
                	print(b0)
                	sumpart <- matrix(0,nrow=p,ncol=p)
                	ee.beta <- matrix(0,nrow=p,ncol=1)

                	scal1 <- mubits1 * (mubits2 - mubits1) - lambda
                	sumpart <- t(x) %*% diag(scal1) %*% x
                	scal2 <- mubits1 - lambda
                	ee.beta <- t(x) %*% scal2

                	ee.beta <- ee.beta/n
                	fish.beta <- -sumpart/n
                	delb2 <- solve(fish.beta) %*% ee.beta
                	delb <- max(abs(as.vector(delb2)))
                	b1 <- b0 + delb2
                	b0 <- b1
        	}
        	lambda <- as.vector(exp(x %*% b1))
		lambda[lambda < 0.001] <- 0.001
		lambda[lambda > 10] <- 10
        	poismu <- lambda * exp(-phi)  #gives an nx1 vector
        	mubits1 <- mubit(poismu,y)
        	mubits2 <- mubit(poismu,(y+1))
	}
	lam <- exp(x %*% b0)
	lam[lam < 0.001] <- 0.001
	lam[lam > 10] <- 10
	phi <- exp(z %*% a0)
	phi[phi > 10] <- 10
	phi[phi < 0.001] <- 0.001
        poismu <- lambda * exp(-phi)  #gives an nx1 vector
        mubits1 <- mubit(poismu,y)
        mubits2 <- mubit(poismu,(y+1))
	sc1 <- as.vector(phi * (phi * mubits1 * (mubits2 - mubits1) - mubits1))
	sumbit1 <- t(z) %*% diag(sc1) %*% z
        sc2 <- as.vector(mubits1 * (mubits2 - mubits1) - lambda)
        sumbit2 <- t(x) %*% diag(sc2) %*% x
	sc3 <- as.vector(-phi * mubits1 * (mubits2 - mubits1))
	sumbit3 <- t(z) %*% diag(sc3) %*% x
	deriv2 <- rbind(cbind(sumbit2,t(sumbit3)),cbind(sumbit3,sumbit1))
	fisher.inv <- -solve(deriv2)	

	list(beta=b0,alpha=a0,covmat=fisher.inv,lambda=lam,phi=phi,
	iter=iter.outer,beta.df=p,alp.df=q)
}

# NEYMAN TYPE A MODEL (1) CALCULATIONS (NULL HYPOTHESIS)
mats <- setup(no.lb~lstags+baa)
y <- as.vector(mats$y)
len <- length(y)
xN <- mats$x
mats2 <- setup(no.lb~slope+shrubs+bark)
zN <- mats2$x
pN <- ncol(xN)
qN <- ncol(zN)
numN <- pN + qN
coefs <- as.vector(slr(y,xN,zN))
b0 <- as.matrix(coefs[1:pN],nrow=pN) 
a0 <- as.matrix(c(coefs[1],coefs[4:6]),nrow=qN) 
estN <- NRalg(y,xN,zN,b0,a0)

beta.se <- as.vector(sqrt(diag(estN$covmat[1:pN,1:pN])))
alpha.se <- as.vector(sqrt(diag(estN$covmat[4:7,4:7])))

beta.tval <- estN$beta/beta.se
beta.tprob <- 1 - pt(abs(beta.tval),(len-pN-qN))
alpha.tval <- estN$alpha/alpha.se
alpha.tprob <- 1 - pt(abs(alpha.tval),(len-pN-qN))
print(cbind(round(estN$beta,4),round(beta.se,4),round(beta.tval,4),round(beta.tprob,4)))
print(cbind(round(estN$alpha,4),round(alpha.se,4),round(alpha.tval,4),round(alpha.tprob,4)))

print(paste(round(min(estN$lambda),4),"to",round(max(estN$lambda),4)))
print(paste(round(min(estN$phi),4),"to",round(max(estN$phi),4)))

loglikN <- loglik.null(estN$lambda,estN$phi,y)


# CALCULATES ESTIMATES OF BETA AND ALPHA, AND HENCE PSI AND PHI, USING THE
# NEWTON-RAPHSON ALGORITHM - NEYMAN TYPE A MODEL (2)
"NRalg" <-
function(y,x,z,b0,a0)
{
        p <- ncol(x)
        q <- ncol(z)
        phi <- as.vector(exp(z %*% a0))
        phi[phi > 10] <- 10
        phi[phi < 0.001] <- 0.001
        psi <- as.vector(exp(x %*% b0))
        psi[psi < 0.001] <- 0.001
        psi[psi > 10] <- 10
        b1 <- matrix(0,nrow=p,ncol=1)
        a1 <- matrix(0,nrow=q,ncol=1)
        tol <- 0.0000001
        maxiter.inner <- 1
        maxiter.outer <- 50
        n <- length(y)
        iter.outer <- 0
        dela <- 1
        delb <- 1
        # CALCULATE POISMU, MUBITS1 AND MUBITS2
        poismu <- psi  #gives an nx1 vector
        mubits1 <- mubit(poismu,y)
        mubits2 <- mubit(poismu,(y+1))

        while (delb > tol & dela > tol & iter.outer < maxiter.outer) {
                iterb <- 0
                itera <- 0
                iter.outer <- iter.outer + 1
                dela <- 1
                delb <- 1

                # ITERATION LOOP FOR ALPHA
                while (dela > tol & itera < maxiter.inner) {
                        itera <- itera + 1
                        print(paste("Starting alpha iteration",iter.outer,"with coefficients"))
                        print(a0)
                        sumpart <- matrix(0,nrow=p,ncol=p)
                        ee.alpha <- matrix(0,nrow=p,ncol=1)

                        scal1 <- -psi * phi * exp(phi) * (phi + 1)
                        sumpart <- t(z) %*% diag(scal1) %*% z
                        scal2 <- y - psi * phi * exp(phi)
                        ee.alpha <- t(z) %*% scal2

                        ee.alpha <- ee.alpha/n
                        fish.alpha <- -sumpart/n
                        dela2 <- solve(fish.alpha) %*% ee.alpha
                        dela <- max(abs(as.vector(dela2)))
                        a1 <- a0 + dela2
                        a0 <- a1
                }
                phi <- as.vector(exp(z %*% a1))
                phi[phi > 10] <- 10
                phi[phi < 0.001] <- 0.001

                # ITERATION LOOP FOR BETA
                while (delb > tol & iterb < maxiter.inner) {
                        iterb <- iterb + 1
                        print(paste("Starting beta iteration",iter.outer,"with coefficients"))
                        print(b0)
                        sumpart <- matrix(0,nrow=p,ncol=p)
                        ee.beta <- matrix(0,nrow=p,ncol=1)

                        scal1 <- mubits1 * (mubits2 - mubits1) - psi * exp(phi)
                        sumpart <- t(x) %*% diag(scal1) %*% x
                        scal2 <- mubits1 - psi * exp(phi)
                        ee.beta <- t(x) %*% scal2

                        ee.beta <- ee.beta/n
                        fish.beta <- -sumpart/n
                        delb2 <- solve(fish.beta) %*% ee.beta
                        delb <- max(abs(as.vector(delb2)))
                        b1 <- b0 + delb2
                        b0 <- b1
                }
                psi <- as.vector(exp(x %*% b1))
                psi[psi < 0.001] <- 0.001
                psi[psi > 10] <- 10
                poismu <- psi  #gives an nx1 vector
                mubits1 <- mubit(poismu,y)
                mubits2 <- mubit(poismu,(y+1))
        }
        psi <- exp(x %*% b0)
        psi[psi < 0.001] <- 0.001
        psi[psi > 10] <- 10
        phi <- exp(z %*% a0)
        phi[phi > 10] <- 10
        phi[phi < 0.001] <- 0.001
        poismu <- psi  #gives an nx1 vector
        mubits1 <- mubit(poismu,y)
        mubits2 <- mubit(poismu,(y+1))
        sc1 <- as.vector(-psi * phi * exp(phi) * (phi + 1))
        sumbit1 <- t(z) %*% diag(sc1) %*% z
        sc2 <- as.vector(mubits1 * (mubits2 - mubits1) - psi * exp(phi))
        sumbit2 <- t(x) %*% diag(sc2) %*% x
        sc3 <- as.vector(-phi * psi * exp(phi))
        sumbit3 <- t(z) %*% diag(sc3) %*% x
        deriv2 <- rbind(cbind(sumbit2,t(sumbit3)),cbind(sumbit3,sumbit1))
        fisher.inv <- -solve(deriv2)

        list(beta=b0,alpha=a0,covmat=fisher.inv,psi=psi,phi=phi,
        iter=iter.outer,beta.df=p,alp.df=q)
}

# NEYMAN TYPE A MODEL (2) CALCULATIONS (ALTERNATIVE HYPOTHESIS)
mats <- setup(no.lb~lstags+baa)
y <- as.vector(mats$y)
len <- length(y)
xA <- mats$x
mats2 <- setup(no.lb~1)
zA <- mats2$x
pA <- ncol(xA)
qA <- ncol(zA)
numA <- pA + qA
coefs <- as.vector(slr(y,xA,zA))
b0 <- as.matrix(c(-5.8472,0.7062,0.0649),nrow=pA)
a0 <- as.matrix(c(1.1517),nrow=qA)
estA <- NRalg(y,xA,zA,b0,a0)

beta.se <- as.vector(sqrt(diag(estA$covmat[1:pA,1:pA])))
alpha.se <- sqrt(estA$covmat[4,4])
cov <- as.matrix(estA$covmat[1:pA,4])

beta.tval <- estA$beta/beta.se
beta.tprob <- 1 - pt(abs(beta.tval),(len-pA-qA))
alpha.tval <- estA$alpha/alpha.se
alpha.tprob <- 1 - pt(abs(alpha.tval),(len-pA-qA))
print(cbind(round(estA$beta,4),round(beta.se,4),round(beta.tval,4),round(beta.tprob,4)))
print(cbind(round(estA$alpha,4),round(alpha.se,4),round(alpha.tval,4),round(alpha.tprob,4)))

print(paste(round(min(estA$psi),4),"to",round(max(estA$psi),4)))
print(paste(round(min(estA$phi),4),"to",round(max(estA$phi),4)))

loglikA <- loglik.alt(estA$psi,estA$phi,y)


# COX'S TEST
est.Lf <- loglikN
est.Lg <- loglikA
exp.LfLg <- exp.fg(estN$lambda,estN$phi,estA$psi,estA$phi,len)
Tf <- est.Lf - est.Lg - exp.LfLg$t1
Tf

var.LfLg <- exp.fg2(estN$lambda,estN$phi,estA$psi,estA$phi,len) - exp.LfLg$t2
expbit <- exp.fgdLf(xN,zN,estN$lambda,estN$phi,estA$psi,estA$phi,len,numN)
varbit <- var.dLf(xN,zN,estN$lambda,estN$phi,len,numN)
Var.Tf <- var.LfLg - t(expbit) %*% solve(varbit) %*% expbit

Zf <- Tf/sqrt(abs(Var.Tf))
Zf
pval <- round(2*(1 - pnorm(abs(Zf))),4)
print(paste("The p-value for testing Neyman type A model (1) vs Neyman type A model (2) is",pval))

detach("counts")
q()

