# ANALYSIS OF POSSUM COUNTS USING THE NEYMAN TYPE A MODEL (3) 

options(contrasts=c("contr.treatment","contr.poly"),object.size=1E8)


# READ IN DATA. SET UP COVARIATES WHICH REQUIRE TRANSFORMATION,FACTORISATION, ETC.
counts <- read.table("data.txt",sep="\t",header=T,row.names=NULL,col.names=c("no.lb","stags","slope","baa","shrubs","bark"))
attach(counts)

lstags <- log(stags+1) 


# SET UP THE RESPONSE AND DESIGN MATRICES FROM AN INPUT FORMULA
"setup"<-
function(formula)
{
	call <- match.call()
        m <- match.call(expand = F)
        m[[1]] <- as.name("model.frame")
        m <- eval(m, sys.parent())
        Terms <- attr(m, "terms")
        y <- as.matrix(model.extract(m, response))
        x <- model.matrix(Terms, m)
	avg.count <- mean(y)
        var.count <- var(y)
        index <- var.count/avg.count
	if(index <= 1) {
		print("Unable to fit the Neyman Distribution")
		stop
	}
	list(y=y,x=x)
}


# CALCULATE INITIAL ESTIMATES FOR THE TWO PARAMETERS, BETA AND ALPHA
"slr" <- 
function(y,x,z)
{
	if (ncol(x) > 1) newx <- as.matrix(x[,-1]) # remove column of ones
	else newx <- x
	if (ncol(z) > 1) newz <- as.matrix(z[,-1]) # remove column of ones
	else newz <- z
	lr <- glm(y~newx+newz,family=poisson,link=log)
	lr$coefficients
}


# CALCULATE THE RATIO OF THE (r+1)th to the rth UNCORRECTED 
# MOMENT OF A POISSON DISTRIBUTION WITH MEAN mu 
"mu1bit" <- 
function(mu,r)
{
# calculation of the (r+1)th uncorrected moment (=mu1)
	delta1 <- 1 #initialisation of maximum increment
        len <- length(mu)
        del1 <- rep(0,len) #initialisation of increment
        j <- 1 #sum counter
        mu1 <- rep(0,len) #initialisation of mu1
        while (delta1 > 10^(-9) | j <= 10) {
                del1 <- exp(-mu)*(mu^j)/gamma(j+1)*(j^(r+1))
                mu1 <- mu1 + del1
                delta1 <- max(abs(del1))
                j <- j + 1
        }

        # calculation of the rth uncorrected moment (=mu0)
        delta0 <- 1 #initialisation of maximum increment
        del0 <- 0 #initialisation of increment
        k <- 1 #sum counter
        mu0 <- rep(0,len) #initialisation of mu1
        for (dd in c(1:len)) {
                if (r[dd]==0) mu0[dd] <- 1
                else {
			while (delta0 > 10^(-9) | k <= 10) {
                                del0 <- exp(-mu[dd])*(mu[dd]^k)/gamma(k+1)*(k^r[dd])
                                mu0[dd] <- mu0[dd] + del0
                                delta0 <- abs(del0)
                                k <- k + 1
			}
                }
                del0 <- 0
                delta0 <- 1
                k <- 1
        }
	mubit <- mu1/mu0
        mubit
}


# CALCULATE THE LOG-LIKELIHOOD OF THE NEYMAN TYPE A MODEL (3)
"loglik" <-
function(eta,phi,y)
{
	n <- length(y)
	sum <- 0
	innersum <- rep(0,n)
	for (i in c(1:n)) {
		j <- 0
		delta <- 1
		if (y[i]==0) innersum[i] <- exp(eta[i]/phi[i])
		else while (delta > 10^(-9) | j <= 10) {
			j <- j + 1
			delta <- (eta[i]/phi[i])^j * (j^y[i]) / gamma(j+1)
			innersum[i] <- innersum[i] + delta
		}
		sum <- sum - eta[i]/(phi[i]*exp(-phi[i])) + y[i] * log(phi[i]) - log(gamma(y[i] + 1)) + 
			log(innersum[i])
	}
	sum
}


# CALCULATE THE LIKELIHOOD-RATIO STATISTIC AND TEST NULL HYPOTHESIS OF PARSIMONIOUS
# MODEL
"lrtest" <-
function(etaA,phiA,etaN,phiN,y,df)
{
	del <- -2 * (loglik(etaN,phiN,y) - loglik(etaA,phiA,y))
	pval <- 1 - pchisq(del,df)
	list(del=del,pval=pval)
}


# CALCULATE ESTIMATES OF BETA AND ALPHA, AND HENCE ETA AND PHI, USING THE
# NEWTON-RAPHSON ALGORITHM
"NRalg" <-
function(y,x,z,b0,a0)
{
	p <- ncol(x)
	q <- ncol(z)
	phi <- as.vector(exp(z %*% a0))
	phi[phi > 10] <- 10
	phi[phi < 0.001] <- 0.001
	eta <- as.vector(exp(x %*% b0))
	eta[eta < 0.001] <- 0.001
	eta[eta > 10] <- 10
	#eta[eta > exp(2)/2] <- 2
	b1 <- matrix(0,nrow=p,ncol=1)
	a1 <- matrix(0,nrow=q,ncol=1)
	tol <- 0.0000001
	maxiter.inner <- 1
	maxiter.outer <- 50
	n <- length(y)
	iter.outer <- 0
	dela <- 1
	delb <- 1
        # CALCULATE POISMU, MUBITS1 AND MUBITS2
        poismu <- eta/phi  #gives an nx1 vector
	#poismu[poismu < 0.001] <- 0.001
	poismu[poismu > 10] <- 10
        mubits1 <- mu1bit(poismu,y)
        mubits2 <- mu1bit(poismu,(y+1))

	while (delb > tol & dela > tol & iter.outer < maxiter.outer) {
		iterb <- 0
		itera <- 0
		iter.outer <- iter.outer + 1
		dela <- 1
        	delb <- 1

        	# ITERATION LOOP FOR ALPHA
        	while (dela > tol & itera < maxiter.inner) {
                	itera <- itera + 1
                	print(paste("Starting alpha iteration",iter.outer,"with coefficients"))
                	print(a0)
                	sumpart <- matrix(0,nrow=p,ncol=p)
                	ee.alpha <- matrix(0,nrow=p,ncol=1)

			phibit <- exp(phi) * ((phi - 1)/phi - phi)
                	scal1 <- eta * phibit + mubits1 * (mubits2 - mubits1)
                	sumpart <- t(z) %*% diag(scal1) %*% z
                	scal2 <- y - mubits1 - eta * exp(phi) * (phi - 1)/phi
                	ee.alpha <- t(z) %*% scal2

                	ee.alpha <- ee.alpha/n
                	fish.alpha <- -sumpart/n
                	dela2 <- solve(fish.alpha) %*% ee.alpha
                	dela <- max(abs(as.vector(dela2)))
                	a1 <- a0 + dela2
                	a0 <- a1
        	}
        	phi <- as.vector(exp(z %*% a1))
		phi[phi > 10] <- 10
		phi[phi < 0.001] <- 0.001
        	poismu <- eta/phi  #gives an nx1 vector
		poismu[poismu > 10] <- 10
		#poismu[poismu < 0.001] <- 0.001
        	mubits1 <- mu1bit(poismu,y)
        	mubits2 <- mu1bit(poismu,(y+1))

        	# ITERATION LOOP FOR BETA
        	while (delb > tol & iterb < maxiter.inner) {
                	iterb <- iterb + 1
                	print(paste("Starting beta iteration",iter.outer,"with coefficients"))
                	print(b0)
                	sumpart <- matrix(0,nrow=p,ncol=p)
                	ee.beta <- matrix(0,nrow=p,ncol=1)

                	scal1 <- mubits1 * (mubits2 - mubits1) - eta * exp(phi)/phi
                	sumpart <- t(x) %*% diag(scal1) %*% x
                	scal2 <- mubits1 - eta * exp(phi)/phi
                	ee.beta <- t(x) %*% scal2

                	ee.beta <- ee.beta/n
                	fish.beta <- -sumpart/n
                	delb2 <- solve(fish.beta) %*% ee.beta
                	delb <- max(abs(as.vector(delb2)))
                	b1 <- b0 + delb2
                	b0 <- b1
        	}
        	eta <- as.vector(exp(x %*% b1))
		eta[eta < 0.001] <- 0.001
		eta[eta > 10] <- 10
        	#eta[eta > exp(2)/2] <- 2
        	poismu <- eta/phi  #gives an nx1 vector
		poismu[poismu > 10] <- 10
		#poismu[poismu < 0.001] <- 0.001
        	mubits1 <- mu1bit(poismu,y)
        	mubits2 <- mu1bit(poismu,(y+1))
	}
	eta <- exp(x %*% b0)
	eta[eta < 0.001] <- 0.001
	eta[eta > 10] <- 10
        #eta[eta > exp(2)/2] <- 2
	phi <- exp(z %*% a0)
	phi[phi > 10] <- 10
	phi[phi < 0.001] <- 0.001
        poismu <- eta/phi  #gives an nx1 vector
	poismu[poismu > 10] <- 10
	#poismu[poismu < 0.001] <- 0.001
        mubits1 <- mu1bit(poismu,y)
        mubits2 <- mu1bit(poismu,(y+1))
	phibit <- as.vector(exp(phi) * ((phi - 1)/phi - phi))
        sc1 <- as.vector(eta * phibit + mubits1 * (mubits2 - mubits1))
        sumbit1 <- t(z) %*% diag(sc1) %*% z
        sc2 <- as.vector(mubits1 * (mubits2 - mubits1) - eta * exp(phi)/phi)
        sumbit2 <- t(x) %*% diag(sc2) %*% x
	sc3 <- as.vector(-eta * exp(phi) * (phi - 1)/phi  - mubits1 * (mubits2 - mubits1))
	sumbit3 <- t(z) %*% diag(sc3) %*% x
	deriv2 <- rbind(cbind(sumbit2,t(sumbit3)),cbind(sumbit3,sumbit1))
	fisher.inv <- -solve(deriv2)	

	list(beta=b0,alpha=a0,covmat=fisher.inv,eta=eta,phi=phi,
	iter=iter.outer,beta.df=p,alp.df=q)
}


# ALTERNATIVE HYPOTHESIS MODEL
mats <- setup(no.lb~lstags+baa+bark)
y <- as.vector(mats$y)
len <- length(y)
xA <- mats$x
mats2 <- setup(no.lb~1)
zA <- mats2$x
p <- ncol(xA)
q <- ncol(zA)
coefs <- as.vector(slr(y,xA,zA))
b0 <- as.matrix(c(coefs[1:p]),nrow=p) 
a0 <- as.matrix(c(coefs[1]),nrow=q) 
estA <- NRalg(y,xA,zA,b0,a0)

beta.se <- as.vector(sqrt(diag(estA$covmat[1:p,1:p])))
alpha.se <- sqrt(estA$covmat[5,5])


# THE FINAL ESTIMATES, STANDARD ERRORS, CORRESPONDING T-RATIOS AND T PROBABILITIES 
# FOR BETA AND ALPHA OF THE ALTERNATIVE HYPOTHESIS MODEL
beta.tval <- estA$beta/beta.se
beta.tprob <- 1 - pt(abs(beta.tval),(len-p-q))
alpha.tval <- estA$alpha/alpha.se
alpha.tprob <- 1 - pt(abs(alpha.tval),(len-p-q))
print(cbind(round(estA$beta,4),round(beta.se,4),round(beta.tval,4),round(beta.tprob,4)))
print(cbind(round(estA$alpha,4),round(alpha.se,4),round(alpha.tval,4),round(alpha.tprob,4)))


# THE RANGE OF ETA AND PHI FOR THE ALTERNATIVE HYPOTHESIS MODELS
print(paste(round(min(estA$eta),4),"to",round(max(estA$eta),4)))
print(paste(round(min(estA$phi),4),"to",round(max(estA$phi),4)))

loglikA <- loglik(estA$eta,estA$phi,y)


# NULL HYPOTHESIS MODEL
mats <- setup(no.lb~lstags+baa)
xN <- mats$x
mats2 <- setup(no.lb~1)
zN <- mats2$x
p <- ncol(xN)
q <- ncol(zN)
coefs <- as.vector(slr(y,xN,zN))
print(coefs)
b0 <- as.matrix(coefs[1:p],nrow=p) 
a0 <- as.matrix(c(coefs[1]),nrow=q) 
estN <- NRalg(y,xN,zN,b0,a0)

beta.se <- as.vector(sqrt(diag(estN$covmat[1:p,1:p])))
alpha.se <- sqrt(estN$covmat[4,4])


# THE FINAL ESTIMATES, STANDARD ERRORS, CORRESPONDING T-RATIOS AND T PROBABILITIES 
# FOR BETA AND ALPHA OF THE NULL HYPOTHESIS MODEL
beta.tval <- estN$beta/beta.se
beta.tprob <- 1 - pt(abs(beta.tval),(len-p-q))
alpha.tval <- estN$alpha/alpha.se
alpha.tprob <- 1 - pt(abs(alpha.tval),(len-p-q))
print(cbind(round(estN$beta,4),round(beta.se,4),round(beta.tval,4),round(beta.tprob,4)))
print(cbind(round(estN$alpha,4),round(alpha.se,4),round(alpha.tval,4),round(alpha.tprob,4)))


# THE RANGE OF ETA AND PHI FOR THE NULL HYPOTHESIS MODELS
print(paste(round(min(estN$eta),4),"to",round(max(estN$eta),4)))
print(paste(round(min(estN$phi),4),"to",round(max(estN$phi),4)))

loglikN <- loglik(estN$eta,estN$phi,y)
loglikN


# TEST THE PARSIMONY OF THE NULL HYPOTHESIS MODEL
df <- estA$beta.df - estN$beta.df + estA$alp.df - estN$alp.df
unlist(lrtest(estA$eta,estA$phi,estN$eta,estN$phi,y,df))


detach("counts")
q()
